require 'mapdata_storage'
require 'mapdata_dbc'
require 'erb'

class Loader
  def Loader.load_wma(path)
      loadfile = WorldMapAreaFile.new(path)
      hash = Hash.new
      loadfile.each do |r|
        var = WorldMapArea.create(r)
        if var.wma_id == 542 
          next
        end
        hash[var.wma_id] = var
      end
      return hash
  end
  def Loader.load_dm(path)
    loadfile = DungeonMapFile.new(path)
    hash = Hash.new
    loadfile.each do |r|
      var = DungeonMap.create(r)
      # Skip floors defined as 0,0 for w,h
      if var.width == 0 and var.height == 0
        next
      end
      hash[var.sub_id] = var
    end
    return hash
  end
  def Loader.load_wmc(path)
    loadfile = WorldMapContinentFile.new(path)
    hash = Hash.new
    loadfile.each do |r|
      var = WorldMapContinent.create(r)
      hash[var.wmc_id] = var
    end
    return hash
  end
  
  def Loader.load_wmaps(path)
    loadfile = MapFile.new(path)
    hash = Hash.new
    loadfile.each do |r|
      var = Map.create(r)
      hash[var.map_id] = var
    end
    return hash    
  end
  
  def Loader.add_floors_to_maps(maps,floors)
      maps.each_key do |map_key|
        map_data = maps[map_key]
        floors.each_value do |floor_info|
          if floor_info.wma_id == map_data.map_id
            if floor_info.is_continent?
              # Dalaran special case
              if floor_info.wma_id == 571 && map_data.area_id == 4395
                map_data.map_floors << floor_info
              end 
            else
              map_data.map_floors << floor_info
            end
          end
        end
        map_data.map_floors.delete_if do |a| a.instance_of?(String) end
        if map_data.map_floors.length > 0
          map_data.map_floors.sort!
          map_data.map_floors.uniq!
        end
        if map_data.map_floors.length == 1 && map_data.map_floors[0].width == map_data.width && map_data.map_floors[0].height == map_data.height
          map_data.map_floors = []
        end
        maps[map_key] = map_data
      end
  end
  
  def Loader.load_all(path)
    world_map_area_entries = load_wma(path)
    dungeon_map_floors = load_dm(path)
    add_floors_to_maps(world_map_area_entries,dungeon_map_floors)
    fullmaps = load_wmaps(path)
    world_map_area_entries.each_key do |wmk|
      data = world_map_area_entries[wmk]
      unless fullmaps[data.map_id].nil?
        data.map_type = fullmaps[data.map_id].map_type
      end
      world_map_area_entries[wmk] = data
    end
    return world_map_area_entries
  end
  
end

path = ARGV[0]
data = Loader.load_all(path)

output = ARGV[1]
output ||= "LibMapData.lua"

# Load ERB template, and process it
 erb_file = IO.read("lib_template.erb")
 template = ERB.new(erb_file,nil, '%><>')
 results = template.result
 fh = File.new("#{output}", "w")
 fh.write(results)
 fh.close